/*
	EZ-USB(AN2135SC) JTAG / CCLK Probe Interface DLL Version 1.10

	Copyright(c) 2002 Tatsuyuki Satoh
	(Refer to GPFW of chamereonUSB by Optimize for EZ-USB firmware and USB controll. )

	2001. 3.18 1st release
	2002. 8.05 Ver.1.00 convert DLL
	2003. 1.16 Ver.1.10 rebuild ALL! , TX-RX firmware , thread for READ , etc.
                        to speed optimize , exsample:playxsvf 8min -> 40sec.
	2003. 3.02 Ver.1.11 bugfix WindowsXP boot probrem.
	                    support 0bits transfer and over 2Mbits trasnfer.
	2003. 4.16 Ver.1.13 feedback ez-usb module from spc_usb2

	support probe
		Cypress AN2135SC(EZ-USB) without EEPROM
		+ Cypress EZ-USB DevKit Device Driver

	I/O port assign for JTAG mode

		FWR      -> TCK
		D[0]     -> TDI
		D[1]     -> TMS
		PC[1]    <- TDO
		PC[3]    <- Target VCC 3.3V

	I/O port assign for SlaveSerial mode

		FWR      -> CCLK
		D[0]     -> DIN
		PC[0]    -> CF_PROGRAM
		PC[1]    <- CF_DONE
		PC[3]    <- Target VCC 3.3V

	Export DLL Functions

	JTAG BounteryScan
		DWORD jtag_version(void)
		int jtag_init(DWORD port_address,DWORD baudrate,char *probe_name)
		int jtag_power(void)
		int jtag_txrx(const BYTE *ibuf,BYTE *obuf,DWORD drsize,int tms_s,int tms_e)
		void jtag_end(void)

	CCLK SerialSlave
		DWORD ss_version(void)
		int ss_init(DWORD port_address,DWORD baudrate,char *probe_name);
		int ss_power(void);
		int ss_tx(const BYTE *data,DWORD size);
		int ss_program(BYTE level);
		int ss_done(void);
		void ss_end(void);

	CCLK ParallelSlave

	note:

	MINGW32 is used for the compiler. 

	The executive speed of EZ-USB is the biggest bottleneck. 
	One bit of JTAGs is transmitted by one byte to simplify the treatment of EZ-USB. 
	And, the core of TX, RX is being written in the assembler. 

	Different Endpoint is used with TX(EP2) only and TX+RX(EP4) by the JTAG transmission. 
	An overlap in pipe switching is avoided as follows. 
	1.TX -> TX+RX : The IRQ priority of EP2 in ezusb is higher than EP4.
	2.TX+RX -> TX : The next TX transmission is transmitted after the RX completion. 

	Thread is used for RX, and TX and RX are carried out at the same time. 
	Because, IOCTRL of the small pieces produces the biggest overhead. 
*/

#include <windows.h>
#include <stdio.h>

// DLL Export prefix
#define DLLEXPORT __declspec(dllexport)
#ifdef __MINGW32__
#define DLLAPI
#else
#define DLLAPI WINAPI
#endif

// Warning fix for EZ-USB header file with MINGW
#if !defined(_BYTE_DEFINED)
#define _BYTE_DEFINED
#define _WORD_DEFINED
#endif

// EZ-USN & chamereonUSB General-purpose FirmWare */
#include "ezusb.h"
#include "jtagfw.h"

/* ---------- Hardware pin assign ---------- */

#define CF_PROGRAM    0x01       /* PC0 */
#define CF_TDI        0x01       /* D0  */
#define CF_DIN        0x01       /* D0  */
#define CF_TMS        0x02       /* D1  */
#define CF_TDO(port)  ((port)&2) /* PC1 */
#define CF_DONE(port) ((port)&2) /* PC1 */
#define CF_VCC(port)  ((port)&8) /* PC3 */

// Pxɓ]őoCg
#define TX_MAX 65536*4 //(2048*1024)
#define TXRX_MAX 64/3

// USB command buffer
static BYTE cmd_buf[64];

// USB FirstWrite buffer
static BYTE usb_buf[TX_MAX];

static int usb_err;

/* -------------------- chamereonUSB General-purpose FirmWare interface -------------------- */

// Write to CPIPE (command)
static int usb_cpipe_write(void *buf,DWORD size)
{
	int ret;

	if(usb_err)
	  return -1;

	ret = ezusb_bulk_write(JTAGFW_CPIPE,buf,size);
	if(ret<0)
	{
		usb_err = 1;
		printf("error ! USB write\n");
	}

	return ret;
}

// Read from RPIPE (read data)
static int usb_rpipe_read(void *buf,DWORD size)
{
	int ret = 0;

	if(usb_err)
	  return -1;

	ezusb_bulk_read(JTAGFW_RPIPE,buf,size);
	//wait for read finish
	if( ezusb_bulk_read_left(TRUE)==0xfffffff)
	{  // error
		ret = -2;
	}

	if(ret<0)
	{
		usb_err = 1;
		printf("error ! USB read\n");
	}
	return size;
}

// Fast WRTITE Transfer
static int usb_fwrite(BYTE gpio,void *data,DWORD size)
{
	int ret;

	if(usb_err)
	  return -1;

	ret = ezusb_bulk_write(JTAGFW_WPIPE,data,size);
	if(ret<0)
		printf("error ! USB FastWrite\n");
	return ret;
	
}

// Fast WRTITE with PORTC Read
static int ezusb_fwrite_read(BYTE gpio,void *data,DWORD size)
{
	int ret;

	if(usb_err)
	  return -1;

	// start read thread for receive TDO data
	ezusb_bulk_read(JTAGFW_RPIPE,data,size);

	// Write TDI data
	ret = ezusb_bulk_write(JTAGFW_DPIPE,data,size);

	//wait for read finish
	if( ezusb_bulk_read_left(TRUE)==0xfffffff)
	{  // error
		ret = -2;
	}

	if(ret<0)
		printf("error ! USB FastRead&Write\n");
	return ret;
	
}

/* -------------------- JTAG contyroll -------------------- */
DLLEXPORT DWORD DLLAPI jtag_version(void)
{
	DWORD mjr = 0x0001;
	DWORD mnr = 0x1300;
	return (mjr<<16) + mnr;
}

DLLEXPORT int DLLAPI jtag_init(DWORD port_address,DWORD baudrate,char *probe_name)
{
	// name set
	if( probe_name != NULL)
	{
		sprintf(probe_name,"EZ-USB No.%d : AN2135SC (FWR=TCK,D0=TDI,D1=TMS,PC1=TDO,PC3=TVCC)"
			,(int)port_address);
	}

	// initialize EZ-USB with JTAG firmware
	usb_err = 1;

	printf("Setup EZ-USB with JTAG firnware V0.09\n");
	if(ezusb_init(port_address,firmware,"JTAG","V009"))
	{
		printf("Can't found EZ-USB\n");
		return 2;
	}
	usb_err = 0;

	// setup GPIO
	cmd_buf[0]=JTAGFW_CDIR;
	cmd_buf[1]=CF_PROGRAM;
	cmd_buf[2]=JTAGFW_CSET;
	cmd_buf[3]=0;
	cmd_buf[4]=JTAGFW_BUS; // streach=0
	cmd_buf[5]= 0x12;    // RMOD = 10 , TMOD = 10
	usb_cpipe_write(cmd_buf,6); // init

	// setup GPIO & read Target VCC port
	cmd_buf[0]=JTAGFW_CGET;
	if( usb_cpipe_write(cmd_buf,1)<0 )
		return 2; /* error */
	if( usb_rpipe_read(cmd_buf,1)<1 )
		return 2; /* error */
	return 0;
}

DLLEXPORT int DLLAPI jtag_power(void)
{
	// setup GPIO & read Target VCC port
	cmd_buf[0]=JTAGFW_CGET;
	if( usb_cpipe_write(cmd_buf,1)<0 )
		return 0; /* error */
	if( usb_rpipe_read(cmd_buf,1)<0 )
		return 0; /* error */

	//Target VCC`FbN
	return CF_VCC(cmd_buf[0]) ? 1 : 0;
}

DLLEXPORT int DLLAPI jtag_txrx(const BYTE *ibuf,BYTE *obuf,DWORD drsize,int tms_s,int tms_e)
{
	int c;
	int pos;
	DWORD block_size;

	while(drsize)
	{
		if(drsize>TX_MAX)
		{
			block_size = TX_MAX;
			memset(usb_buf,tms_s ? 0x02 : 0x00,TX_MAX);
		}
		else
		{
			block_size = drsize;
			// TMS̒lobt@ɍ쐬
			memset(usb_buf,tms_s ? 0x02 : 0x00,drsize-1);
			usb_buf[drsize-1] = tms_e ? CF_TMS : 0x00;
		}

		// TDI̒lobt@ɍ쐬
		if(ibuf)
		{
			for(pos=0 ; pos<block_size ; pos+=8)
			{
				BYTE value = ibuf[pos/8];
				// nibble -> bitϊ , obt@쐬
				if(value&0x80) usb_buf[pos+0] |= CF_TDI;
				if(value&0x40) usb_buf[pos+1] |= CF_TDI;
				if(value&0x20) usb_buf[pos+2] |= CF_TDI;
				if(value&0x10) usb_buf[pos+3] |= CF_TDI;
				if(value&0x08) usb_buf[pos+4] |= CF_TDI;
				if(value&0x04) usb_buf[pos+5] |= CF_TDI;
				if(value&0x02) usb_buf[pos+6] |= CF_TDI;
				if(value&0x01) usb_buf[pos+7] |= CF_TDI;
			}
		}

		if(obuf == NULL)
			//M
			usb_fwrite(0,usb_buf,block_size);
		else
		{
			// M
			ezusb_fwrite_read(0,usb_buf,block_size);
			// TDOMf[^̃oCgϊ
			memset(obuf,0,(block_size+7)/8);
			for(c=0;c<block_size;c++)
			{
				if( CF_TDO(usb_buf[c]) )
					obuf[c/8]  |= 0x01<<(c&7);
			}
		}
		// ̃ubN
		if(ibuf)
			ibuf   += (block_size/8);
		if(obuf)
			obuf   += (block_size/8);
		drsize -= block_size;
	}
	return 1;
}

DLLEXPORT void DLLAPI jtag_end(void)
{
	ezusb_close();
}

/* -------------------- CCLK contyroll -------------------- */
//return version
DLLEXPORT int DLLAPI ss_version(void)
{
	DWORD mjr = 0x01;
	DWORD mnr = 0x10;
	return (mjr<<16) + mnr;
}

// initialize PROBE
DLLEXPORT int DLLAPI ss_init(DWORD port_address,DWORD baudrate,char *probe_name)
{
	// name set
	if( probe_name != NULL)
	{
		sprintf(probe_name,"EZ-USB No.%d : AN2135SC (FWR=CCLK,D0=DIN,PC0=PROGRAM,PC1=DONE,PC3=TVCC)"
		,(int)port_address);
	}
	return jtag_init(port_address,baudrate,NULL);
}

DLLEXPORT int DLLAPI ss_power(void)
{
	return jtag_power();
}

DLLEXPORT int DLLAPI ss_tx(const BYTE *data,DWORD size)
{
	DWORD pos;

	// PubN̑Mf[^쐬
	for(pos=0 ; pos<size ; pos++)
	{
		BYTE value = data[pos];
		// nibble -> bitϊ , obt@쐬
		if(value&0x80) usb_buf[pos*8+0] |= CF_DIN;
		if(value&0x40) usb_buf[pos*8+1] |= CF_DIN;
		if(value&0x20) usb_buf[pos*8+2] |= CF_DIN;
		if(value&0x10) usb_buf[pos*8+3] |= CF_DIN;
		if(value&0x08) usb_buf[pos*8+4] |= CF_DIN;
		if(value&0x04) usb_buf[pos*8+5] |= CF_DIN;
		if(value&0x02) usb_buf[pos*8+6] |= CF_DIN;
		if(value&0x01) usb_buf[pos*8+7] |= CF_DIN;
	}
	//PubNAo[Xg]B
	usb_fwrite(CF_PROGRAM,usb_buf,size*8);
	return 0;
}

// set PROGRAM pin level
DLLEXPORT int DLLAPI ss_program(BYTE level)
{
	// PROGRAMskɂB
	cmd_buf[0] = JTAGFW_CSET;
	cmd_buf[1] = level ? CF_PROGRAM : 0x00;
	usb_cpipe_write(cmd_buf,2);
	return 0;
}

// return DONE pin level
DLLEXPORT int DLLAPI ss_done(void)
{
	cmd_buf[0]=JTAGFW_CGET;
	usb_cpipe_write(cmd_buf,1);
	usb_rpipe_read(cmd_buf,1);
	return CF_DONE(cmd_buf[0]) ? 1 : 0;
}

DLLEXPORT void DLLAPI ss_end(void)
{
	ezusb_close();
}
