#ifndef __JTAGFW_H__
#define __JTAGFW_H__
/*
	EZ_SPI       EZ-USB Synchronus Serial Interface Firmware.
	Copyright(C) 2003 Tatsuyuki Satoh.

	--- speed ---
	The executive speed of EZ-USB is the biggest bottleneck. 
	The serial-I/F is used with the special trick to full-duplex transmission.
	The clock rate in byte transfer mode is 2MHz(Typ. 1.2Mbps @ Burst TX).
	The core routine of TX, RX is being written in the assembler. 

	--- synchronize ---
	Different Endpoint is used with TX(EP2OUT) only and TX+RX(EP4/EP6) by the JTAG transmission. 
	An overlap in pipe access switching is avoided as follows. 
	1.TX -> TX+RX : The IRQ priority of EP2 in ezusb is higher than EP4.
	2.TX+RX -> TX : The next TX start after the RX completion of last TX.

	
	Use thread for RX,and TX and RX are carried out at the same time. 
	Because, IOCTRL of the small length produces the biggest overhead. 

	--- device driver ---
	The Vecder ID and Product ID are default value.
	The standard development driver of Cypress "ezusb.sys" can be use for it.

	---------- EZ-USB AN2131SC/QC,AN2135SC pin assign ----------
	+--------+----+-----+-----+-----+
	|Port    |Dir | SPI |JTAG |MMC*1|
	+--------+----+-----+-----+-----+
	|PA5     | -> | TXD | TDI | CMD |
	|PC1/TXD | -> | CLK | TCK | CLK |
	|PC0/RXD | <- | RXD | TDO | DAT |
	|PC2     | -> | CS  | TMS | CS  |
	+--------+----+-----+-----+-----+
	|PC3     | <- | VCC sense       |
	|PC4     | <- | input pin       |
	|PC5     | <- | input pin       |
	|PC6     | <- | input pin       |
	|PC7     | <- | input pin       |
	|(other) |    | Don't care      |
	+--------+----+-----------------+
	*1:MMC/SD Card with SPI mode.

	---------- PIPE / EndPoint Assign ----------
	+-----+----------+----------------------------+
	|PIPE | EndPoint | Function                   |
	+-----+----------+----------------------------+
	|  0  | EP1OUT   | EZ-USB memory access       |
	|  1  | EP2OUT   | Byte Transmit only         |
	|  2  | EP4OUT   | Byte Transmit and receive  |
	|  3  | EP6OUT   | Bit Transmit and receive   |
	|  5  | EP2IN    | EP4OUT,EP6OUT response     |
	|other| other    | Don't cate                 |
	+-----+----------+----------------------------+

	---------- Byte access Format (EP2.OUT , EP4.OUT) ----------
	Byte Access

	Transmit Data , LSB first.
	When EP4OUT access then return receviced Data to EP2IN

	---------- Bit access Format (EP6.OUT ) ----------

	1.one bit access
	   +----+----+----+----+----+----+----+----+
	   |bit7|bit6|bit5|bit4|bit3|bit2|bit1|bit0|
	   +----+----+----+----+----+----+----+----+
	TX | -- | -- |TXD | -- | -- |CS  | 0  | -- | EP6OUT
	   +----+----+----+----+----+----+----+----+
	RX |PC7 |PC6 |PC5 |PC4 |*PC3| -- | -- |RXD | EP2IN
	   +----+----+----+----+----+----+----+----+
		1.direct out PortA and PortC from EP6OUT data.
		2.set CLK(PC1) to High.
		3.return PortC input to EP2IN.

	2.set port and sense port
	   +----+----+----+----+----+----+----+----+
	   |bit7|bit6|bit5|bit4|bit3|bit2|bit1|bit0|
	   +----+----+----+----+----+----+----+----+
	TX | -- | -- |TXD | -- | -- | CS | 1  | -- | EP2OUT / EP4OUT
	   +----+----+----+----+----+----+----+----+
	RX |PC7 |PC6 |PC5 |PC4 |PC3 | -- | -- |RXD | EP2IN
	   +----+----+----+----+----+----+----+----+
		same as bit access mode but, no CLK assert

*/

/* USB PIPE assign */
#define SPI_CPIPE     0 /* Ep1OUT : EZ-USB Memory access                */
#define SPI_BYTE_TX   1 /* Ep2OUT : SPI BYTE Trnasmit only              */
#define SPI_BYTE_TXRX 2 /* Ep4OUT : SPI BYTE Transmit and Receive       */
#define SPI_PROTCOL   3 /* Ep6OUT : SPI Mixed TX & RX and I/O controll  */
#define SPI_STATUS    4 /* Ep1IN  : response of CPIPE                   */
#define SPI_RESPONSE  5 /* Ep2IN  : response of TX & RX                 */
#define SPI_EP4IN     6 /* Ep4IN  : free                                */
#define SPI_EP6IN     7 /* Ep6IN  : free                                */

extern unsigned char ez_spi[];
#endif

