/*
	EZ-USB(AN2131SC,AN2135SC) JTAG / CCLK Probe Interface DLL Version 1.00

	Copyright(c) 2002,2003 Tatsuyuki Satoh

	2003. 4.16 Ver. 1.0000
	           Created from spc_usb.c
	           support AN2131SC and AN2135SC with hardware SPI.
	           Port Assign are changed.

	supported probe and device driver
		Cypress EZ-USB (AN2131SC or AN2135SC) without EEPROM
		+ Cypress EZ-USB DevKit Device Driver

	I/O port assign for JTAG mode

		PC0/RXD  <- TDO (pull
		PC1/TXD  -> TCK
		PC2      -> TMS
		PA5      -> TDI
		PC3      <- Target VCC 3.3V (100Kohm pull down)

	Export DLL Functions

	JTAG BounteryScan
		DWORD jtag_version(void)
		int jtag_init(DWORD port_address,DWORD baudrate,char *probe_name)
		int jtag_power(void)
		int jtag_txrx(const BYTE *ibuf,BYTE *obuf,DWORD drsize,int tms_s,int tms_e)
		void jtag_end(void)

	note:

	MINGW32 2.95.2 is used for the compile. 
	see ez_spi.h to check the specification.

*/

#include <windows.h>
#include <stdio.h>

// DLL Export prefix
#define DLLEXPORT __declspec(dllexport)
#ifdef __MINGW32__
#define DLLAPI
#else
#define DLLAPI WINAPI
#endif

// Warning fix for EZ-USB header file with MINGW
#if !defined(_BYTE_DEFINED)
#define _BYTE_DEFINED
#define _WORD_DEFINED
#endif

// EZ-USN & chamereonUSB General-purpose FirmWare */
#include "ezusb.h"
#include "ez_spi.h"

/* ---------- Hardware pin assign ---------- */

/* JTAG mode */
#define CF_TDO(port)  ((port)&0x01) /* PC0/RXD : Can't reassign */
#define CF_TCK        0x02          /* PC1/TXD : Can't reassign */
#define CF_TMS        0x04          /* PC2     : Can reassign   */
#define CF_TDI        0x20          /* PA5     : Can't reassign */
#define CF_TVCC(port) ((port)&0x08) /* PC3     : Can reassign   */

/* SS mode */
#define CF_CCLK       CF_TCK
#define CF_PROGRAM    CF_TMS
#define CF_DIN        CF_TDI
#define CF_DONE(port) CF_TDO(port)

/* bit swap table */
static BYTE bit_swap[256];

/* USB transmit buffer */
#define TX_MAX 32768
static BYTE usb_buf[TX_MAX];

static int last_tms;


/* USB Static Error Flag */
static int usb_err;

/* -------------------- EZ-USB SPI Inteface -------------------- */

static int spi_tx(const void *txd,DWORD size)
{
	int ret;

	if(usb_err)
	  return -1;

	ret = ezusb_bulk_write(SPI_BYTE_TX,(void *)txd,size);
	if(ret<0)
		printf("error ! USB SPI TX\n");
	return ret;
}

static int spi_txrx(const void *txd,void *rxd,DWORD size)
{
	int ret;

	if(usb_err)
	  return -1;

	/* start read thread for receive data */
	ezusb_bulk_read(SPI_RESPONSE,rxd,size);

	ret = ezusb_bulk_write(SPI_BYTE_TXRX,(void *)txd,size);

	//wait for read finish
	if( ezusb_bulk_read_left(TRUE) == 0xfffffff)
	{  // error
		ret = -2;
	}

	if(ret<0)
		printf("error ! USB SPI TX-RX\n");
	return ret;
}


/* I/O port,bit and byte mixed mode */
static int spi_protcol_xfer(void *data,DWORD size)
{
	int ret;

	if(usb_err)
	  return -1;

	// start read thread for receive TDO data
	ezusb_bulk_read(SPI_RESPONSE,data,size);

	// Write TDI data
	ret = ezusb_bulk_write(SPI_PROTCOL,data,size);

	//wait for read finish
	if( ezusb_bulk_read_left(TRUE) == 0xfffffff)
	{  // error
		ret = -2;
	}

	if(ret<0)
		printf("error ! USB SPI Protcol\n");
	return ret;
}

/* -------------------- JTAG contyroll -------------------- */
DLLEXPORT DWORD DLLAPI jtag_version(void)
{
	DWORD mjr = 0x0001;
	DWORD mnr = 0x0000;
	return (mjr<<16) + mnr;
}

DLLEXPORT int DLLAPI jtag_init(DWORD port_address,DWORD baudrate,char *probe_name)
{
	int i;

	for(i=0;i<256;i++)
	{
		bit_swap[i]  = (i&0x01)<<7;
		bit_swap[i] |= (i&0x02)<<5;
		bit_swap[i] |= (i&0x04)<<3;
		bit_swap[i] |= (i&0x08)<<1;
		bit_swap[i] |= (i&0x10)>>1;
		bit_swap[i] |= (i&0x20)>>3;
		bit_swap[i] |= (i&0x40)>>5;
		bit_swap[i] |= (i&0x80)>>7;
	}

	// name set
	if( probe_name != NULL)
	{
		sprintf(probe_name,"EZ-USB No.%d : SPI(PC1=TCK,PA5=TDI,PC2=TMS,PC0=TDO,PC3=TVCC)"
			,(int)port_address);
	}

	// initialize EZ-USB with JTAG firmware
	usb_err = 1;

	printf("Setup EZ-USB Firmware\n");
	if(ezusb_init(port_address,ez_spi,"USB_SPI","V010"))
	{
		printf("\nCan't found EZ-USB\n");
		return 2;
	}
	printf("\n");
	usb_err = 0;

	last_tms = -1;

	return 0;
}

DLLEXPORT int DLLAPI jtag_power(void)
{
	// no TCK access
	usb_buf[0] = CF_TMS | CF_TCK;
	if( spi_protcol_xfer(usb_buf,1) )
		return 0; /* error */

	//Target VCC`FbN
	return CF_TVCC(usb_buf[0]) ? 1 : 0;
}

//#define BLEN_MAX 0x7f
#define BLEN_MAX 0x50


DLLEXPORT int DLLAPI jtag_txrx(const BYTE *ibuf,BYTE *obuf,DWORD drsize,int tms_s,int tms_e)
{
	int pos ,rblen , bits;
	DWORD block_size;
	DWORD byte_len;
	BYTE d;
	int dummy_byte;

	if(drsize==0)
		return 0;

	/* byte transer mode */
	rblen = 0;
	dummy_byte = 0;

	/* remove last bit for BYTE command */
	if(tms_s != tms_e)
		drsize--;

	/* BYTE transfer mode */
	if(drsize>=8)
	{
		if(last_tms != tms_s)
		{
			last_tms = tms_s;
			/* update TMS level */
			usb_buf[0] = tms_s ? (CF_TMS|CF_TCK) : CF_TCK;
			spi_protcol_xfer(usb_buf,1);
		}

		while(drsize>=8)
		{
			block_size = drsize/8;
			if(block_size>TX_MAX)
				block_size = TX_MAX;

			/* setup TXD */
			if(ibuf)
			{
				for(byte_len=0;byte_len<block_size;byte_len++)
					usb_buf[byte_len] = bit_swap[ibuf[byte_len]];
				ibuf += block_size;
			}
			else
			{
				memset(usb_buf,0x00,block_size);
			}

			/* TX or TX&RX */
			if(!obuf)
				spi_tx(usb_buf,block_size);
			else
			{
				spi_txrx(usb_buf,obuf,block_size);
				obuf += block_size;
			}
			drsize -= block_size*8;
		}
	}

	/* restore last bit */
	if(tms_s != tms_e)
		drsize++;

	/* get last byte */
	d = ibuf ? *ibuf : 0x00;

	/* BIT mode in last byte */
	block_size = 0;
	while(drsize--)
	{
		/* TMS level */
		if(drsize==0)
			tms_s = tms_e;
		/* BYTE command with TMS level */
		usb_buf[block_size] = tms_s ? CF_TMS : 0x00;
		/* TDI level */
		if(d&0x80)
			usb_buf[block_size] |= CF_TDI;
		block_size++;
		d<<=1;
	}

	/* flush bit and last bite */
	if(block_size)
	{
		if(obuf==NULL)
		{	/* TX only */
			spi_protcol_xfer(usb_buf,block_size);
		}
		else
		{	/* TX & RX */
			spi_protcol_xfer(usb_buf,block_size);
			rblen = 0;
			bits = 0x01;
			/* RX data & increment pointer */
			for(pos=0;pos<block_size;pos++)
			{
				if(rblen)
				{
					*obuf++ = usb_buf[pos];
					rblen--;
				}
				else
				{
					//if(usb_buf[pos]&0x80)
					//	rblen = usb_buf[pos] & 0x7f;
					//else
					{	/* get bit data */
						if(bits == 0x01)
							*obuf = 0x00;
						if(CF_TDO(usb_buf[pos]))
							*obuf |= bits;
						bits <<= 1;
					}
				}
			}
		}
	}

	last_tms = tms_e;

	return 1;
}

DLLEXPORT void DLLAPI jtag_end(void)
{
	ezusb_close();
}

